/*******************************************************************************

  Defines Filter Plug-in Library Interface for DIMIN Viewer n5 (DIV5)

  Programmer: Dima V. Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  Plug-in Notes:
    Each DLL must export (following plugin defs):
      1) divGetPluginHeader
    Each filter DLL must export:
      1) divGetFilterHeader
      2) divGetFilter
      3) define pluginType in TDIVPluginHeader as DIV_PLUGIN_TYPE_FILTER
      4) define count with correct number of filters present

  Filter naming:
    Dimin Image Viewer naming follows:
    DIV_CHANNEL_
    DIV_CHANNELS_
    DIV_COLOR_
    DIV_HISTOGRAM_
    DIV_EFFECT_
    DIV_FILTER_
    DIV_IMAGE_

    Other names are ecoureged to separate third party filters and embedded ones.
    If plug-in filter name copies the embedded name then it will be used instead. 

  Notes:
  - v5 is not compatible with previous v2, since nobody used it, it's ok :-)
  - Compatibility with mc++ is maintained now throughout whole structure

  History:
    12/05/2000 20:36:48 - First Creation
    03/25/2001 15:45:51 - TDIVParameters, TDIVFilterHeader
    03/11/2002 17:14:13 - v 2.2
    06/19/2004 16:28:00 - v 5
    09/12/2004 23:44:00 - v 5.1
    10/23/2004 14:46:00 - v 5.2
    04/24/2005 17:13:00 - v 5.3
    07/10/2005 22:46:00 - v5.4 changed TDivConfigProc and TDivAboutProc
    09/28/2005 18:23:00 - v5.5 changed void* into a union, for static float defs

  Version : 5.5

*******************************************************************************/

#ifndef DIV_FILTER_DEFS_H
#define DIV_FILTER_DEFS_H

#include <windows.h>
#include "DIVTypeDefs.h"

#include "DIVFilterDefs.h"

#define DIV_PLUGIN_FILTER_ENTRY_NAME "divGetFilterHeader"
#define DIV_PLUGIN_FILTER_PROCS_NAME "divGetFilter"


//****************************************************************************
// Filter arguents
//****************************************************************************

union TDIVDataValue
{
  DIM_INT32  iVal;  // int value
  DIM_UINT32 uVal;  // int value
  DIM_FLOAT  fVal;  // float value
  void      *pVal;  // pointer value
  TDIVDataValue ()          { }
  TDIVDataValue (DIM_INT32   tmp) { iVal = tmp; }
  TDIVDataValue (DIM_UINT32  tmp) { uVal = tmp; }
  TDIVDataValue (DIM_FLOAT   tmp) { fVal = tmp; }
  TDIVDataValue (void       *tmp) { pVal = tmp; }
  TDIVDataValue (char       *tmp) { pVal = tmp; }
  TDIVDataValue (const char *tmp) { pVal = (void *) tmp; }
};

typedef struct TDIVTagedArgument {
  DWORD tagId;           // id number, plugin may use ids above 1000 as private
  DWORD tagType;         // TDimTagType
  DWORD tagCount;        // number of records of type tagType pointed by tagData
  char  *tagDescr;       // text description of the argument
  TDIVDataValue tagData; // pointer to data, if data size is <= 4 bytes
                         // then it's directly stored into pointer
  TDIVDataValue tagMin;  // pointer to min val, used if needed to define
                         // a special value retrieval, e.g. -100 .. 100
  TDIVDataValue tagMax;  // pointer to max val
  TDIVTagedArgument()
  {
  }
  TDIVTagedArgument(DWORD _tagId, DWORD _tagType, DWORD _tagCount, char  *_tagDescr, TDIVDataValue _tagData,
    TDIVDataValue _tagMin, TDIVDataValue _tagMax)
  {
    tagId    = _tagId;
    tagType  = _tagType;
    tagCount = _tagCount;
    tagDescr = _tagDescr;
    tagData  = _tagData;
    tagMin   = _tagMin;
    tagMax   = _tagMax;
  }
} TDIVTagedArgument;

// callback provided by filter in case of custom controls creation
// if plugin provides this function then it's called and PDivArgumentArray
// is not used for creating components in the window, it's used to pass
// parameters thought, but the TDivArgsInitProc should modify it
// if callback proc is NULL and there are arguments, they will be read
// by automatically created procedure, if there are no arguments or array
// is null, nothing is done... 


// updated version 5.4:
struct TDIVParameters;

typedef void (__stdcall *TDivConfigProc) 
                ( HWND hwndParent, // handle to a scroll box window where filter can create it's
                                   // own controls such as buttons, check boxes using CreateWindow
                                  // (or CreateParented in case of Borland VCL)
                  TDIVParameters *pars
                );

typedef void (__stdcall *TDivAboutProc) ( HWND hwndParent, TDIVParameters *pars );


typedef struct TDIVArguments {
  DWORD usePreview;            // each byte meaning and states:
                               // 1th: preview (0-no, 1-open)
                               // 2nd: changes (0-image, 1-file, 2-none)
                               // 3td: reserved (0)
                               // 4th: reserved (0)
    // thus values mean:
    //0 - no preview and changes image,
    //1 - open preview and changes image
    //256 - no preview and changes file
    //512 - no preview and no changes
    //257 - preview and changes file
    //513 - preview and no changes
  DWORD count;                 // number of arguments in the array
  TDIVTagedArgument *args;     // pointer to array of argumants
  TDivConfigProc confInitProc; // custom controls creation
  TDivConfigProc confQuitProc; // called when parent window is being closed
  void *Param1;                // reserved 
} TDIVArguments;


//****************************************************************************
// Plugin Headers
//****************************************************************************

typedef struct TDIVFilterHeader {
  int             ver;         // Size of whole structure
  char           *shortName;   // desirably unique name for the filter e.g. DIV_ROTATE
  char           *description; // Text about filter
  void           *filterProc;  // pointer to a TPluginFilter function
  int             bppNeed;     // Bpp needed by filter: 1,4,8,24,32
                               // or: 0-any, 888-Gray, 111-mono, 666-Gray or 24, 777-8or24
  TDIVArguments  *args;        // arguments, if NULL equals to usePreview = 0
  TDivAboutProc   aboutProc;   // about proc provided by filter
} TDIVFilterHeader;


//****************************************************************************
// Callback filters forward declaration
//****************************************************************************

//typedef struct TDIVCallbackFilters;
struct TDIVCallbackFilters;


//****************************************************************************
// Callbacks
//****************************************************************************

// host shows progress of the executon
// done  - value in a range of [0..total]
// total - range maximum
// descr - any text describing the executed procedure
typedef void (__stdcall *TDivProgressProc)(long done, long total, char *descr);

// host shows error of the executon
// val   - error code, not treated at the moment
// descr - any text describing the error
typedef void (__stdcall *TDivErrorProc)(int val, char *descr);

// plugin should use this to test is processing should be interrupted...
// If it returns TRUE, the operation should be aborted 
typedef int (__stdcall *TDivTestAbortProc)( void );


//****************************************************************************
// Selection
//****************************************************************************

#if defined(__BORLANDC__)
namespace DIV5 {
  typedef struct TPoint
  {
    LONG x;
    LONG y;
  } TPoint;
}
  typedef struct TDIVSelection {
    RECT   rectangle;  // Selected rectangle
    DWORD  chainCount; // size of chain array
    DIV5::TPoint *chain;     // pointer to array of TPoint of size chainCount
  } TDIVSelection;
#else
  namespace DIV5 {
  }

  typedef struct TPoint
  {
    LONG x;
    LONG y;
  } TPoint;

  typedef struct TDIVSelection {
    RECT   rectangle;  // Selected rectangle
    DWORD  chainCount; // size of chain array
    TPoint *chain;     // pointer to array of TPoint of size chainCount
  } TDIVSelection;
#endif



//****************************************************************************
// Main Structure
//****************************************************************************

#pragma pack(push, 8)
typedef struct TDIVParameters {
  
  DWORD ver;      // Size of the whole structure
  
  // image part
  HBITMAP  hbmp;  // handle to BITMAP structure
  HPALETTE hpal;  // handle to PALETTE structure
  char *fileName; // pointer to a complete file name

  // callbacks
  TDivProgressProc  progressProc;
  TDivErrorProc     errorProc;
  TDivTestAbortProc testAbortProc;  

  DWORD curX, curY;        // position selected by user on image
  TDIVSelection selection; // shape of selected region  

  TDIVArguments *args;     // arguments, if NULL equals to count = 0  
  TDIVCallbackFilters *functions; // Filters implemented in DIV

  #if defined(__BORLANDC__)
  #endif
  void *chainList;         // a pointer to TPointList (borland stuff)
  
  HWND      hwndParent;    // main window handle
  HINSTANCE hDllInstance;  // handle to loaded dll instance (plug-in)

  void *Param1;            // reserved 
  void *Param2;            // reserved 
  void *Param3;            // reserved 
  char reserved[32];
} TDIVParameters;
#pragma pack(pop)


//****************************************************************************
// Filter DLL Functions
//****************************************************************************

// Next functions must to be exported by DLL to be accepted as valid
// Note : parametr passage is by you, send wrong program may hang !!!
     
typedef int (__stdcall *TDivPluginFilter)( TDIVParameters *par );

// divGetFilterHeader
typedef TDIVFilterHeader* (__stdcall *TDivGetFilterHeader)(int I);
// divGetFilter
typedef TDivPluginFilter  (__stdcall *TDivGetFilter)(int I);


//****************************************************************************
// Callback filters
//****************************************************************************

typedef struct TDIVCallbackFilters {
  DWORD             ver; // Size of whole structure
  DWORD             numFilters; // number of filters in the array
  TDIVFilterHeader *filterArray; // array of filter headers
  unsigned char     reserved[32];
  
} TDIVCallbackFilters;


#endif //DIV_FILTER_DEFS_H
